/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.data;
import static nuts.util.CollUtils.list;

import java.io.File;
import java.util.List;
import java.util.Random;
import java.util.Set;

import nuts.util.CollUtils;
import nuts.util.Indexer;
import fig.basic.Option;
import fig.basic.Pair;
import fig.exec.Execution;
import gep.comparisons.TargetModel;
import gep.pmcmc.Generator;
import gep.timeseries.Measurements;


public class GeneratedData implements DataSource
{
  @Option public Random dataRandom = new Random(1);
  @Option public Random trueParamRandom = new Random(1);
  
  @Option public double samplingDelta = 1.0;
  @Option public int nDataPtPerSeries = 10;
  
  @Option public int nStates = 10;
  @Option public int obsSize = 4;
  
  @Option public double transitionDensity = 0.2;
  @Option public double smallRate = 0.01;
  @Option public double largeSmallRateDiff = 0.5;
  
  private TargetModel _model = null;
  private Set<String> _possibleObs = null;
  
  private TargetModel model()
  {
    if (_model != null) return _model;
    _model = TargetModel.randomTargetType1(
        trueParamRandom, 
        nStates, 
        obsSize, 
        transitionDensity, 
        smallRate, 
        largeSmallRateDiff);
    _model.save(new File(Execution.getFile("model")));
    return _model;
  }
  
  @Override
  public Measurements next(Indexer<String> indexer)
  {
    Pair<List<Integer>,List<Integer>> generated = Generator.generate(dataRandom, model(), nDataPtPerSeries, samplingDelta);
    List<Integer> states = generated.getSecond(); // first is hid, second is observed
    List<Double> times = list();
    double currentTime = 0.0;
    for (int currentSqIdx = 0; currentSqIdx < states.size(); currentSqIdx++)
    {
      times.add(currentTime);
      currentTime += samplingDelta;
    }
    return new Measurements(times, states);
  }

  @Override
  public Set<String> possibleObservations()
  {
    if (_possibleObs != null) return _possibleObs;
    _possibleObs = CollUtils.set();
    for (int i = 0; i < obsSize; i++)
      _possibleObs.add("obs" + i);
    return _possibleObs;
  }
}