/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.data;
import java.io.*;
import java.util.*;

import pepper.Encodings;
import pty.RootedTree;
import pty.io.Dataset;
import pty.learn.DiscreteBP;
import pty.smc.models.CTMC;

import fig.basic.LogInfo;
import fig.basic.Option;
import gep.timeseries.Measurements;
import goblin.Taxon;
import ma.MSAParser;
import ma.MSAPoset;
import ma.RateMatrixLoader;
import ma.SequenceType;
import nuts.io.IO;
import nuts.math.GMFct;
import nuts.util.CollUtils.*;
import nuts.util.Arbre;
import nuts.util.CollUtils;
import nuts.util.Counter;
import nuts.util.Indexer;
import static nuts.util.CollUtils.*;
import static nuts.io.IO.*;
import static nuts.util.MathUtils.*;


public class MSData implements DataSource
{
  @Option public String dataFile = "";
//  @Option public Random rand = new Random(1);
//  @Option public double timeScale = 26.0; 
  @Option public double timeScale = 1.0;
  
  private List<Measurements> _data = null;
  private int pointer = 0;
  
  
  private void ensureLoaded(Indexer<String> indexer)
  {
    if (_data != null) return;
    _data = CollUtils.list();
//    createIndexer();
    
    int index = 0;
    List<Double> times = list();
    List<Integer> values = list();
    for (String line : IO.i(dataFile))
    {
      if (index % 2 == 0)
      {
        // reading first of pair (time)
        for (String item : line.split("\\s+"))
          if (!item.equals("NA"))
          {
            double cur = Double.parseDouble(item);
            times.add(cur/timeScale);
          }
      }
      else
      {
        // reading second (values)
        for (String item : line.split("\\s+"))
          if (!item.equals("NA"))
          {
            if (item.length() != 1)
              throw new RuntimeException();
            int currentIdx = indexer.o2i("" + item.charAt(0));
            values.add(currentIdx);
//            values.add(Integer.parseInt(item));
          }
        
        Measurements currentM = new Measurements(times, values);
//        System.out.println(currentM);
        _data.add(currentM);
        times = list();
        values = list();
      }
      index++;
    }
    

//    Collections.shuffle(_data, rand); BAD! DONE IN GEPMAIN INSTEAD
  }

  @Override
  public Measurements next(Indexer<String> indexer)
  {
    ensureLoaded(indexer);
    if (pointer++ < _data.size())
      return _data.get(pointer-1);
    else
      return null;
  }
  
  private Set<String> _possibleObs;
  
  @Override
  public Set<String> possibleObservations()
  {
    if (_possibleObs == null)
      createSet();
    return _possibleObs;
  }
  private void createSet()
  {
    _possibleObs = CollUtils.set();
//    _indexer.addToIndex('0');
    _possibleObs.add("1");
    _possibleObs.add("2");
    _possibleObs.add("3");
//    _indexer.addToIndex('4');
//    _indexer.addToIndex('5');
//    return result;
  }
  
}
