/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.model;

import nuts.util.Indexer;


public class SplitContext
{
  public static final SplitContext BEG = new SplitContext(-5,-5);
  public static final SplitContext TOP = new SplitContext(-1,-1);
  
  private final double u;
  private final int x;
  public double getU()
  {
    if (u < 0 || u > 1)
      throw new RuntimeException();
    return u;
  }
  public int getX() 
  {
    if (x < 0)
      throw new RuntimeException();
    return x;
  }
  public int index(Indexer<Double> indexer)
  {
    if (u < 0) throw new RuntimeException();
    indexer.addToIndex(u);
    return indexer.o2i(u);
  }
  @Override
  public String toString()
  {
    if (this.equals(TOP)) return "TOP";
    if (this.equals(BEG)) return "BEG";
    String result = "[" + x;
    result += "/" + ( u <0 ? "*" : "" +u ) + "]";
    return result;
  }
  public String toString(Indexer<Double> hiddenIndexer, Indexer<String> observedIndexer)
  {
    if (this.equals(TOP)) return "TOP";
    if (this.equals(BEG)) return "BEG";
    String result = "[" + observedIndexer.i2o(x);
    result += "/" + ( u <0 ? "*" : "" +index(hiddenIndexer) ) + "]";
    return result;
  }
  public int level()
  {
    if      (u == -1 && x == -1) return 2;
    else if (u == -1 && x >= 0)  return 1;
    else if (u >= 0 && x >= 0)   return 0;
    else if (this.equals(BEG))   return 1;
    else 
      throw new RuntimeException();
  }
  public SplitContext backoff()
  {
    final int level = level();
    if      (level == 2)   return null; // you are at top, signals will have to use base measure
    else if (level == 1)   return TOP;
    else if (level == 0)   return new SplitContext(-1, x);
    else throw new RuntimeException();
  }
  public SplitContext(int x)
  {
    this(-1, x);
  }
  public SplitContext(double u, int x) 
  {
    if (u >= 0 && x == -1) 
      throw new RuntimeException();
    this.x = x;
    this.u = u;
  }
  @Override
  public int hashCode()
  {
    final int prime = 31;
    int result = 1;
    long temp;
    temp = Double.doubleToLongBits(u);
    result = prime * result + (int) (temp ^ (temp >>> 32));
    result = prime * result + x;
    return result;
  }
  @Override
  public boolean equals(Object obj)
  {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    SplitContext other = (SplitContext) obj;
    if (Double.doubleToLongBits(u) != Double.doubleToLongBits(other.u))
      return false;
    if (x != other.x)
      return false;
    return true;
  }
}