/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.pmcmc;
import static nuts.util.CollUtils.list;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import nuts.math.RateMtxUtils;
import nuts.util.EasyFormat;
import nuts.util.Indexer;
import pepper.Encodings;
import fig.basic.Pair;
import fig.basic.StrUtils;
import fig.prob.SampleUtils;
import gep.GEPMain;
import gep.comparisons.TargetModel;
import gep.model.Predictives;
import gep.model.SplitContext;
import gep.model.SufficientStatistics;
import gep.model.SufficientStatisticsImpl;
import gep.timeseries.Event;


public class Generator
{
  public static double MAX_INTERVAL = 100;
  /**
   * @param add suff stats used both for pred. NOT WRITTEN TO!
   * @param pred side effect there
   * @param rand
   * @param eventsList a list of event that are used both as prefix/context (if empty, first event will be generated from BEG as previous), and to carry the generated events 
   * @param delta the amount of time after which to stop adding events
   * @param maxN limit the number of transitions (incl self) to generate
   * @param begSymbol the begin symbol to use
   * @return
   */
  
  
  public static <S> SufficientStatistics<S,S> generate(
      SufficientStatistics<S,S> init, 
      Predictives<S,S> pred, 
      Random rand, 
      List<Event<S>> eventsList, 
      final double delta,
      final int maxN,
      S begSymbol)
  {
    


    
    SufficientStatistics<S,S> result = new SufficientStatisticsImpl<S,S>(init);
    final double initRight = eventsList.isEmpty() ? 0.0 : eventsList.get(eventsList.size()-1)._right;
    double currentDelta = 0.0;
    int cur = 0;
//    if (eventsList.isEmpty())
//      ;
    while (currentDelta <= delta &&  cur < maxN)  // cur++ < maxN) WEIRD: if uncomment this and comment next stmt, creates bug (seems like a bug in mac JVM 1.6.0_26)
    {
      cur++;
      
      // sample:
      SufficientStatistics<S,S> current 
        = new SufficientStatisticsImpl<S,S>(result);
      Pair<S,Double> newSample = null;
      S prev = eventsList.isEmpty() ? begSymbol : eventsList.get(eventsList.size()-1)._contents;
      if (eventsList.isEmpty())
        newSample = Pair.makePair(pred.sampleState(rand, prev, current), 0.0);
      else
        newSample = pred.sample(rand, prev, current);
      // avoid internal BEG symbols
      if (newSample.equals(SplitContext.BEG) || newSample.getSecond() > MAX_INTERVAL)
        reject++; // reject!
      else
      {
        accept++;
        final double curLeft = currentDelta+initRight,
                     curRight= curLeft + newSample.getSecond();
        currentDelta += newSample.getSecond();
        eventsList.add(new Event<S>(curLeft, curRight, newSample.getFirst()));
        result = current; 
      }
      
      
    }
    
    
    return result;
  }
  
  public static double reject = 0.0, accept = 0.0;

  public static SufficientStatistics<SplitContext,SplitContext> generate(
      Predictives<SplitContext,SplitContext> pred, 
      Random rand, 
      List<Event<SplitContext>> eventsList, 
      double delta,
      int maxN)
  {
    return generate(new SufficientStatisticsImpl<SplitContext,SplitContext>(), pred, rand, eventsList, delta, maxN, SplitContext.BEG);
  }

  
  public static void main(String [] args)
  {
    test();
//    Random rand = new Random(1);
//    TargetModel model = randomTargetType1(rand, 10, 4, 0.2, 0.01, 0.5);
//    System.out.println(model);
  }
  
  
  public static void test()
  {
    Indexer<Character> observedIndexer = Encodings.rnaEncodings().nonGapCharactersIndexer();
    Indexer<Double> hiddenIndexer = new Indexer<Double>();
    double bmAlpha0 = 100, alphaL1 = 100, alphaL2 = 100, alphaL3 = 100, beta0 = 100, gamma0 = 100;
    Predictives<SplitContext,SplitContext> pred = 
      Predictives.splitPredictives(
          new SufficientStatisticsImpl<SplitContext,SplitContext>(), 
          bmAlpha0, 
          alphaL1, 
          alphaL2, 
          alphaL3, 
          4, 
          beta0, gamma0);
    List<Event<SplitContext>> eventsList = list();
    Random rand = new Random(1);
    generate(pred, rand, eventsList, 100, 100);
//    System.out.println(toString(eventsList, hiddenIndexer,observedIndexer ));
  }
  
  public static String toString(
      List<Event<SplitContext>> seqn, 
      Indexer<Double> hiddenIndexer, 
      Indexer<String> observedIndexer)
  {
    StringBuilder result = new StringBuilder();
    for (int i = 1; i < seqn.size(); i++)
    {
      Event<SplitContext> e1 = seqn.get(i-1), e2 = seqn.get(i);
      result.append(Event.contents(e1,e2).toString(hiddenIndexer, observedIndexer) + "@[" + EasyFormat.fmt2(Event.left(e1,e2)) + "," 
          + EasyFormat.fmt2(Event.right(e1,e2)) + ")\n");
    }
    return result.toString();
  }
  
  // generate from delta-squeleton
  // (list of hidden, list of obs)
  public static Pair<List<Integer>,List<Integer>> generate(Random rand, TargetModel model, int maxL, double delta)
  {
    List<Integer> obss =list(), hids = list(); 
    double [][] trans = RateMtxUtils.marginalTransitionMtx(model.transitionRates, delta);
    int hidden = SampleUtils.sampleMultinomial(rand, RateMtxUtils.getStationaryDistribution(model.transitionRates));
    for (int i = 0; i < maxL; i++)
    {
      int obs = SampleUtils.sampleMultinomial(rand, model.emiPrs[hidden]);
      obss.add(obs);
      hids.add(hidden);
      hidden = SampleUtils.sampleMultinomial(rand, trans[hidden]);
    }
    return Pair.makePair(hids, obss);
  }
  
  
  public static <S> SufficientStatistics<S,S> generateHitting(
	      SufficientStatistics<S,S> init, 
	      Predictives<S,S> pred, 
	      Random rand, 
	      //List<Event<S>> eventsList, 
	      double maxT,
	      int maxN,
	      S begSymbol)
	  {
	  
	    List<Event<S>>  eventsList = new ArrayList<Event<S>> (3);
	    eventsList.clear();
	    SufficientStatistics<S,S> result = new SufficientStatisticsImpl<S,S>(init);
//	    final double initRight = eventsList.isEmpty() ? 0.0 : eventsList.get(eventsList.size()-1)._right;
	    final double initRight = 0.0;
	    double currentDelta = 0.0;
	    int cur = 0;
	    int begSt = 0;
	    Event<S> e1 , e2 ;
	    S sc = null;
	 // sample:
	      SufficientStatistics<S,S> current 
	        = new SufficientStatisticsImpl<S,S>(result);
	      Pair<S,Double> newSample = null;
	      S prev =  begSymbol ;
	      newSample = Pair.makePair(pred.sampleState(rand, prev, current), 0.0);
	     
	      // avoid internal BEG symbols
	      if (newSample.equals(SplitContext.BEG) || newSample.getSecond() > MAX_INTERVAL)
	        reject++; // reject!
	      else
	      {
	        accept++;
	        final double curLeft = currentDelta+initRight,
	                     curRight= curLeft + newSample.getSecond();
	        currentDelta += newSample.getSecond();
	        eventsList.add(new Event<S>(curLeft, curRight, newSample.getFirst()));
            e1 = eventsList.get(eventsList.size()-1);
            sc = Event.tempcontents(e1);
            begSt = ((SplitContext) sc).getX();
	        result = current; 
	      }
	    while (((SplitContext) sc).getX() != 3 && cur++ < maxN && currentDelta < maxT)
	    {
	      // sample:
	      current 
	        = new SufficientStatisticsImpl<S,S>(result);
	       newSample = null;
	      prev = eventsList.isEmpty() ? begSymbol : eventsList.get(eventsList.size()-1)._contents;
	      if (eventsList.isEmpty())
	        newSample = Pair.makePair(pred.sampleState(rand, prev, current), 0.0);
	      else
	        newSample = pred.sample(rand, prev, current);
	      // avoid internal BEG symbols
	      if (newSample.equals(SplitContext.BEG) || newSample.getSecond() > MAX_INTERVAL )
	        reject++; // reject!
	      else
	      {
	        accept++;
	        final double curLeft = currentDelta+initRight,
	                     curRight= curLeft + newSample.getSecond();
	        currentDelta += newSample.getSecond();
	        eventsList.add(new Event<S>(curLeft, curRight, newSample.getFirst()));
	        e1 = eventsList.get(eventsList.size()-1);
            sc = Event.tempcontents(e1);
//            System.out.println(((SplitContext) sc).getX());
	        result = current; 
	      }
	    }
	    GEPMain.outputManager.printWrite("hittingTime", "Time", currentDelta, "BegState", begSt, "FinalState", ((SplitContext) sc).getX());
	    return result;
	  }
  
  public static SufficientStatistics<SplitContext,SplitContext> generateHitting(
	      Predictives<SplitContext,SplitContext> pred, 
	      Random rand, 
	      List<Event<SplitContext>> eventsList, 
	      double delta,
	      int maxN)
	  {
	    return generateHitting(new SufficientStatisticsImpl<SplitContext,SplitContext>(), pred, rand, delta, maxN, SplitContext.BEG);
	  }
}
