/**
 * http://www.stat.ubc.ca/~bouchard/GEP/README-usage.html
 * 
 * @author Alexandre Bouchard
 * @author Ardavan Saeedi
 */
package gep.timeseries;

import static nuts.util.MathUtils.checkClose;


public class Event<S>
{
  public final double _right;
  public final double _left;
  public final S _contents; // NOTE: DANGEROUS: contents only starts at right.. see methods below to avoid danger
  public Event(double left, double right, S contents)
  {
    if (right < left || left < 0 || Double.isNaN(left) || Double.isNaN(right))
      throw new RuntimeException();
    _left = left;
    _right = right;
    _contents = contents;
  }
  public double right() { return _right;}
  public double delta() { return _right - _left; }
  
  @Override
  public String toString()
  {
    return "[" + _left + "," + _right + "," + _contents + "]";
  }
  
  /*
   * If you care about a pair (contents, interval), where
   * interval = [left,right) and
   * the contents is present in the full interval, use the methods below,
   * where e1, e2 are two successive events
   */
  public static <S> double left(Event<S> e1, Event<S> e2)
  {
    checkClose(e1._right,e2._left);
    return e2._left;
  }
  public static <S> double right(Event<S> e1, Event<S> e2)
  {
    checkClose(e1._right,e2._left);
    return e2._right;
  }
  public static <S> S contents(Event<S> e1, Event<S> e2)
  {
    checkClose(e1._right,e2._left);
    return e1._contents;
  }
  public static <S> S tempcontents(Event<S> e1)
  {
    return e1._contents;
  }
}